@@ -1,11 +1,11 @@
-import os
 import random
 import weakref
 
-from redis.client import StrictRedis
-from redis.connection import ConnectionPool, Connection
-from redis.exceptions import ConnectionError, ResponseError, ReadOnlyError
-from redis._compat import iteritems, nativestr, xrange
+from redis.client import Redis
+from redis.commands import SentinelCommands
+from redis.connection import Connection, ConnectionPool, SSLConnection
+from redis.exceptions import ConnectionError, ReadOnlyError, ResponseError, TimeoutError
+from redis.utils import str_if_bytes
 
 
 class MasterNotFoundError(ConnectionError):
@@ -18,26 +18,26 @@ class SlaveNotFoundError(ConnectionError):
 
 class SentinelManagedConnection(Connection):
     def __init__(self, **kwargs):
-        self.connection_pool = kwargs.pop('connection_pool')
-        super(SentinelManagedConnection, self).__init__(**kwargs)
+        self.connection_pool = kwargs.pop("connection_pool")
+        super().__init__(**kwargs)
 
     def __repr__(self):
         pool = self.connection_pool
-        s = '%s<service=%s%%s>' % (type(self).__name__, pool.service_name)
+        s = f"{type(self).__name__}<service={pool.service_name}%s>"
         if self.host:
-            host_info = ',host=%s,port=%s' % (self.host, self.port)
+            host_info = f",host={self.host},port={self.port}"
             s = s % host_info
         return s
 
     def connect_to(self, address):
         self.host, self.port = address
-        super(SentinelManagedConnection, self).connect()
+        super().connect()
         if self.connection_pool.check_connection:
-            self.send_command('PING')
-            if nativestr(self.read_response()) != 'PONG':
-                raise ConnectionError('PING failed')
+            self.send_command("PING")
+            if str_if_bytes(self.read_response()) != "PONG":
+                raise ConnectionError("PING failed")
 
-    def connect(self):
+    def _connect_retry(self):
         if self._sock:
             return  # already connected
         if self.connection_pool.is_master:
@@ -50,9 +50,15 @@ class SentinelManagedConnection(Connection):
                     continue
             raise SlaveNotFoundError  # Never be here
 
-    def send_command(self, *args):
+    def connect(self):
+        return self.retry.call_with_retry(
+            self._connect_retry,
+            lambda error: None,
+        )
+
+    def read_response(self, disable_decoding=False):
         try:
-            super(SentinelManagedConnection, self).send_command(*args)
+            return super().read_response(disable_decoding=disable_decoding)
         except ReadOnlyError:
             if self.connection_pool.is_master:
                 # When talking to a master, a ReadOnlyError when likely
@@ -61,10 +67,14 @@ class SentinelManagedConnection(Connection):
                 # calling disconnect will force the connection to re-query
                 # sentinel during the next connect() attempt.
                 self.disconnect()
-                raise ConnectionError('The previous master is now a slave')
+                raise ConnectionError("The previous master is now a slave")
             raise
 
 
+class SentinelManagedSSLConnection(SentinelManagedConnection, SSLConnection):
+    pass
+
+
 class SentinelConnectionPool(ConnectionPool):
     """
     Sentinel backed connection pool.
@@ -74,36 +84,43 @@ class SentinelConnectionPool(ConnectionPool):
     """
 
     def __init__(self, service_name, sentinel_manager, **kwargs):
-        kwargs['connection_class'] = kwargs.get(
-            'connection_class', SentinelManagedConnection)
-        self.is_master = kwargs.pop('is_master', True)
-        self.check_connection = kwargs.pop('check_connection', False)
-        super(SentinelConnectionPool, self).__init__(**kwargs)
-        self.connection_kwargs['connection_pool'] = weakref.proxy(self)
+        kwargs["connection_class"] = kwargs.get(
+            "connection_class",
+            SentinelManagedSSLConnection
+            if kwargs.pop("ssl", False)
+            else SentinelManagedConnection,
+        )
+        self.is_master = kwargs.pop("is_master", True)
+        self.check_connection = kwargs.pop("check_connection", False)
+        super().__init__(**kwargs)
+        self.connection_kwargs["connection_pool"] = weakref.proxy(self)
         self.service_name = service_name
         self.sentinel_manager = sentinel_manager
 
     def __repr__(self):
-        return "%s<service=%s(%s)" % (
-            type(self).__name__,
-            self.service_name,
-            self.is_master and 'master' or 'slave',
-        )
+        role = "master" if self.is_master else "slave"
+        return f"{type(self).__name__}<service={self.service_name}({role})"
 
     def reset(self):
-        super(SentinelConnectionPool, self).reset()
+        super().reset()
         self.master_address = None
         self.slave_rr_counter = None
 
+    def owns_connection(self, connection):
+        check = not self.is_master or (
+            self.is_master and self.master_address == (connection.host, connection.port)
+        )
+        parent = super()
+        return check and parent.owns_connection(connection)
+
     def get_master_address(self):
-        master_address = self.sentinel_manager.discover_master(
-            self.service_name)
+        master_address = self.sentinel_manager.discover_master(self.service_name)
         if self.is_master:
-            if self.master_address is None:
+            if self.master_address != master_address:
                 self.master_address = master_address
-            elif master_address != self.master_address:
-                # Master address changed, disconnect all clients in this pool
-                self.disconnect()
+                # disconnect any idle connections so that they reconnect
+                # to the new master the next time that they are used.
+                self.disconnect(inuse_connections=False)
         return master_address
 
     def rotate_slaves(self):
@@ -112,9 +129,8 @@ class SentinelConnectionPool(ConnectionPool):
         if slaves:
             if self.slave_rr_counter is None:
                 self.slave_rr_counter = random.randint(0, len(slaves) - 1)
-            for _ in xrange(len(slaves)):
-                self.slave_rr_counter = (
-                    self.slave_rr_counter + 1) % len(slaves)
+            for _ in range(len(slaves)):
+                self.slave_rr_counter = (self.slave_rr_counter + 1) % len(slaves)
                 slave = slaves[self.slave_rr_counter]
                 yield slave
         # Fallback to the master connection
@@ -122,19 +138,10 @@ class SentinelConnectionPool(ConnectionPool):
             yield self.get_master_address()
         except MasterNotFoundError:
             pass
-        raise SlaveNotFoundError('No slave found for %r' % (self.service_name))
-
-    def _checkpid(self):
-        if self.pid != os.getpid():
-            self.disconnect()
-            self.reset()
-            self.__init__(self.service_name, self.sentinel_manager,
-                          connection_class=self.connection_class,
-                          max_connections=self.max_connections,
-                          **self.connection_kwargs)
+        raise SlaveNotFoundError(f"No slave found for {self.service_name!r}")
 
 
-class Sentinel(object):
+class Sentinel(SentinelCommands):
     """
     Redis Sentinel cluster client
 
@@ -144,7 +151,7 @@ class Sentinel(object):
     >>> master.set('foo', 'bar')
     >>> slave = sentinel.slave_for('mymaster', socket_timeout=0.1)
     >>> slave.get('foo')
-    'bar'
+    b'bar'
 
     ``sentinels`` is a list of sentinel nodes. Each node is represented by
     a pair (hostname, port).
@@ -163,38 +170,60 @@ class Sentinel(object):
     establishing a connection to a Redis server.
     """
 
-    def __init__(self, sentinels, min_other_sentinels=0, sentinel_kwargs=None,
-                 **connection_kwargs):
+    def __init__(
+        self,
+        sentinels,
+        min_other_sentinels=0,
+        sentinel_kwargs=None,
+        **connection_kwargs,
+    ):
         # if sentinel_kwargs isn't defined, use the socket_* options from
         # connection_kwargs
         if sentinel_kwargs is None:
-            sentinel_kwargs = dict([(k, v)
-                                    for k, v in iteritems(connection_kwargs)
-                                    if k.startswith('socket_')
-                                    ])
+            sentinel_kwargs = {
+                k: v for k, v in connection_kwargs.items() if k.startswith("socket_")
+            }
         self.sentinel_kwargs = sentinel_kwargs
 
-        self.sentinels = [StrictRedis(hostname, port, **self.sentinel_kwargs)
-                          for hostname, port in sentinels]
+        self.sentinels = [
+            Redis(hostname, port, **self.sentinel_kwargs)
+            for hostname, port in sentinels
+        ]
         self.min_other_sentinels = min_other_sentinels
         self.connection_kwargs = connection_kwargs
 
+    def execute_command(self, *args, **kwargs):
+        """
+        Execute Sentinel command in sentinel nodes.
+        once - If set to True, then execute the resulting command on a single
+        node at random, rather than across the entire sentinel cluster.
+        """
+        once = bool(kwargs.get("once", False))
+        if "once" in kwargs.keys():
+            kwargs.pop("once")
+
+        if once:
+            for sentinel in self.sentinels:
+                sentinel.execute_command(*args, **kwargs)
+        else:
+            random.choice(self.sentinels).execute_command(*args, **kwargs)
+        return True
+
     def __repr__(self):
         sentinel_addresses = []
         for sentinel in self.sentinels:
-            sentinel_addresses.append('%s:%s' % (
-                sentinel.connection_pool.connection_kwargs['host'],
-                sentinel.connection_pool.connection_kwargs['port'],
-            ))
-        return '%s<sentinels=[%s]>' % (
-            type(self).__name__,
-            ','.join(sentinel_addresses))
+            sentinel_addresses.append(
+                "{host}:{port}".format_map(
+                    sentinel.connection_pool.connection_kwargs,
+                )
+            )
+        return f'{type(self).__name__}<sentinels=[{",".join(sentinel_addresses)}]>'
 
     def check_master_state(self, state, service_name):
-        if not state['is_master'] or state['is_sdown'] or state['is_odown']:
+        if not state["is_master"] or state["is_sdown"] or state["is_odown"]:
             return False
         # Check if our sentinel doesn't see other nodes
-        if state['num-other-sentinels'] < self.min_other_sentinels:
+        if state["num-other-sentinels"] < self.min_other_sentinels:
             return False
         return True
 
@@ -209,23 +238,25 @@ class Sentinel(object):
         for sentinel_no, sentinel in enumerate(self.sentinels):
             try:
                 masters = sentinel.sentinel_masters()
-            except ConnectionError:
+            except (ConnectionError, TimeoutError):
                 continue
             state = masters.get(service_name)
             if state and self.check_master_state(state, service_name):
                 # Put this sentinel at the top of the list
                 self.sentinels[0], self.sentinels[sentinel_no] = (
-                    sentinel, self.sentinels[0])
-                return state['ip'], state['port']
-        raise MasterNotFoundError("No master found for %r" % (service_name,))
+                    sentinel,
+                    self.sentinels[0],
+                )
+                return state["ip"], state["port"]
+        raise MasterNotFoundError(f"No master found for {service_name!r}")
 
     def filter_slaves(self, slaves):
         "Remove slaves that are in an ODOWN or SDOWN state"
         slaves_alive = []
         for slave in slaves:
-            if slave['is_odown'] or slave['is_sdown']:
+            if slave["is_odown"] or slave["is_sdown"]:
                 continue
-            slaves_alive.append((slave['ip'], slave['port']))
+            slaves_alive.append((slave["ip"], slave["port"]))
         return slaves_alive
 
     def discover_slaves(self, service_name):
@@ -233,52 +264,67 @@ class Sentinel(object):
         for sentinel in self.sentinels:
             try:
                 slaves = sentinel.sentinel_slaves(service_name)
-            except (ConnectionError, ResponseError):
+            except (ConnectionError, ResponseError, TimeoutError):
                 continue
             slaves = self.filter_slaves(slaves)
             if slaves:
                 return slaves
         return []
 
-    def master_for(self, service_name, redis_class=StrictRedis,
-                   connection_pool_class=SentinelConnectionPool, **kwargs):
+    def master_for(
+        self,
+        service_name,
+        redis_class=Redis,
+        connection_pool_class=SentinelConnectionPool,
+        **kwargs,
+    ):
         """
         Returns a redis client instance for the ``service_name`` master.
 
-        A SentinelConnectionPool class is used to retrive the master's
-        address before establishing a new connection.
+        A :py:class:`~redis.sentinel.SentinelConnectionPool` class is
+        used to retrieve the master's address before establishing a new
+        connection.
 
         NOTE: If the master's address has changed, any cached connections to
         the old master are closed.
 
-        By default clients will be a redis.StrictRedis instance. Specify a
-        different class to the ``redis_class`` argument if you desire
-        something different.
+        By default clients will be a :py:class:`~redis.Redis` instance.
+        Specify a different class to the ``redis_class`` argument if you
+        desire something different.
 
-        The ``connection_pool_class`` specifies the connection pool to use.
-        The SentinelConnectionPool will be used by default.
+        The ``connection_pool_class`` specifies the connection pool to
+        use.  The :py:class:`~redis.sentinel.SentinelConnectionPool`
+        will be used by default.
 
         All other keyword arguments are merged with any connection_kwargs
         passed to this class and passed to the connection pool as keyword
         arguments to be used to initialize Redis connections.
         """
-        kwargs['is_master'] = True
+        kwargs["is_master"] = True
         connection_kwargs = dict(self.connection_kwargs)
         connection_kwargs.update(kwargs)
-        return redis_class(connection_pool=connection_pool_class(
-            service_name, self, **connection_kwargs))
+        return redis_class(
+            connection_pool=connection_pool_class(
+                service_name, self, **connection_kwargs
+            )
+        )
 
-    def slave_for(self, service_name, redis_class=StrictRedis,
-                  connection_pool_class=SentinelConnectionPool, **kwargs):
+    def slave_for(
+        self,
+        service_name,
+        redis_class=Redis,
+        connection_pool_class=SentinelConnectionPool,
+        **kwargs,
+    ):
         """
         Returns redis client instance for the ``service_name`` slave(s).
 
-        A SentinelConnectionPool class is used to retrive the slave's
+        A SentinelConnectionPool class is used to retrieve the slave's
         address before establishing a new connection.
 
-        By default clients will be a redis.StrictRedis instance. Specify a
-        different class to the ``redis_class`` argument if you desire
-        something different.
+        By default clients will be a :py:class:`~redis.Redis` instance.
+        Specify a different class to the ``redis_class`` argument if you
+        desire something different.
 
         The ``connection_pool_class`` specifies the connection pool to use.
         The SentinelConnectionPool will be used by default.
@@ -287,8 +333,11 @@ class Sentinel(object):
         passed to this class and passed to the connection pool as keyword
         arguments to be used to initialize Redis connections.
         """
-        kwargs['is_master'] = False
+        kwargs["is_master"] = False
         connection_kwargs = dict(self.connection_kwargs)
         connection_kwargs.update(kwargs)
-        return redis_class(connection_pool=connection_pool_class(
-            service_name, self, **connection_kwargs))
+        return redis_class(
+            connection_pool=connection_pool_class(
+                service_name, self, **connection_kwargs
+            )
+        )
